import React, { HTMLAttributes, MutableRefObject } from 'react';
import * as _foundationui_react_hooks from '@foundationui/react-hooks';
import { useFoundationModel } from '@foundationui/react-hooks';
import * as slate from 'slate';
import { BaseEditor, Operation, Range, Descendant } from 'slate';
import * as slate_react from 'slate-react';
import { ReactEditor } from 'slate-react';

/**
 * The shape of the completion response from the API.
 */
interface ICompletionResponse {
    sid: string;
    input: string;
    completion?: string | null;
}
type CustomElement = {
    type: 'text' | 'completion' | undefined;
    children: CustomText[];
};
type CustomText = {
    text: string;
    type: undefined;
};
type CustomDescendant = CustomElement | CustomText;
type CEditor = Omit<BaseEditor, 'children'> & ReactEditor & {
    children: CustomDescendant[];
};
declare module 'slate' {
    interface CustomTypes {
        Editor: CEditor;
        Element: CustomElement;
        Text: CustomText;
    }
}

interface Batch {
    operations: Operation[];
    selectionBefore: Range | null;
}
/**
 * `History` objects hold all of the operations that are applied to a value, so
 * they can be undone or redone as necessary.
 */
interface History {
    redos: Batch[];
    undos: Batch[];
}
declare const History: {
    /**
     * Check if a value is a `History` object.
     */
    isHistory(value: any): value is History;
};

/**
 * `HistoryEditor` contains helpers for history-enabled editors.
 */
interface HistoryEditor extends CEditor {
    history: History;
    undo: () => void;
    redo: () => void;
}
declare const HistoryEditor: {
    /**
     * Check if a value is a `HistoryEditor` object.
     */
    isHistoryEditor(value: any): value is HistoryEditor;
    /**
     * Get the merge flag's current value.
     */
    isMerging(editor: HistoryEditor): boolean | undefined;
    /**
     * Get the saving flag's current value.
     */
    isSaving(editor: HistoryEditor): boolean | undefined;
    /**
     * Redo to the previous saved state.
     */
    redo(editor: HistoryEditor): void;
    /**
     * Undo to the previous saved state.
     */
    undo(editor: HistoryEditor): void;
    /**
     * Apply a series of changes inside a synchronous `fn`, without merging any of
     * the new operations into previous save point in the history.
     */
    withoutMerging(editor: HistoryEditor, fn: () => void): void;
    /**
     * Apply a series of changes inside a synchronous `fn`, without saving any of
     * their operations into the history.
     */
    withoutSaving(editor: HistoryEditor, fn: () => void): void;
};

/**
 * Responsible for fetching completions from a model and adding them to the input.
 * Each `<SmartInput />` component rendered to the DOM will have its own instance of this class as state.
 */
declare class SmartInputFetcher {
    addCompletion: (text: string, completion: ICompletionResponse, force?: boolean) => void;
    foundationModel: ReturnType<typeof useFoundationModel>;
    model: string;
    /**
     * sequenceId is used to ensure that we don't add completions from a previous fetch,
     * we increment it each time we request to fetch a new completion because the user input has changed
     */
    sequenceId: number;
    /**
     * sessionId is used to track this particular instantiation of the `<SmartInput />` component
     */
    sessionId: string;
    /**
     * completionCache is used to cache the completions that we've already fetched from the model
     */
    completionCache: Record<string, ICompletionResponse | null>;
    /**
     * currentCompletion tracks the latest text that was sent to the model for fetching completions, and the
     * completion that was returned (if any ~ we may still be waiting for it)
     */
    currentCompletion: {
        sequenceId: number;
        text: string;
        completion?: string | null;
    } | null;
    blurred: boolean;
    ignoreNextFetch: boolean;
    eventsByModel: Record<string, {
        model: string;
        session: string;
        input: string;
        user?: string;
        context?: string;
    }>;
    constructor(addCompletion: (text: string, completion: ICompletionResponse, force?: boolean) => void, foundationModel: ReturnType<typeof useFoundationModel>, model: string);
    fetchCompletion: {
        (...args: any[]): any;
        cancel: () => void;
        flush: () => any;
    };
    updateCompletion(completionSid: string, arg: {
        seen?: boolean;
        accepted?: boolean;
    }): void;
    addEvent: {
        (...args: any[]): any;
        cancel: () => void;
        flush: () => any;
    };
}

declare function useSmartInput(arg: {
    model: string;
    value: string;
    onAcceptCompletion?: (completion: string) => void;
    onShowCompletion?: (completion: string) => void;
    context?: string;
    user?: string;
    dev?: boolean;
    safeMode?: boolean;
}): {
    editor: Omit<slate.BaseEditor, "children"> & slate_react.ReactEditor & {
        children: CustomDescendant[];
    } & HistoryEditor;
    foundationModel: {
        model: string;
        user: string | undefined;
        saveCompletion: (input: string, context?: string | undefined) => Promise<number | void>;
        getCompletion: (input: string, context?: string | undefined) => Promise<_foundationui_react_hooks.ICompletionResponse | null>;
        markCompletionSeen: (completionSid: string) => void;
        markCompletionAccepted: (completionSid: string) => void;
    };
    completion: ICompletionResponse | null;
    setCompletion: React.Dispatch<React.SetStateAction<ICompletionResponse | null>>;
    fetcher: SmartInputFetcher;
    onChange: (value: Descendant[]) => string;
    triggerCompletion: () => void;
    addCompletion: (completion: ICompletionResponse, force?: boolean) => void;
    acceptCompletion: () => void;
    clearCompletion: () => void;
};

/**
 * Properties for the `SmartInput` component.
 */
interface SmartInputProps<P> extends Omit<HTMLAttributes<HTMLElement>, 'onChange'> {
    smartInputHooks?: ReturnType<typeof useSmartInput>;
    /**
     * The path of the model to use for completions, e.g. "my-org/my-input".
     */
    model: string;
    /**
     * The ID of the user using the input. Providing this helps the model learn a particular user's style,
     * including if they prefer not to use completions at all.
     */
    user?: string;
    /**
     * Data that will help the model generate better completions.
     * Pass anything that you think is relevant to making the completions better or more personalized.
     * For example, it could be an English sentence description of what's happening in your app or a JSON.stringify'd object
     * of useful key/value pairs.
     */
    context?: string;
    /**
     * A ref to the underlying contenteditable div. This can be used to focus the editor, select text, etc.
     */
    /**
     * The controlled value of the input.
     */
    value: string;
    /**
     * Called whenever the value of the input changes. Use this to update the controlled `value` property.
     */
    onChange: (newValue: string) => void;
    /**
     * Renders text the user has input.
     */
    renderText: (props: any) => React.ReactElement<any, any>;
    /**
     * Renders suggested, tab-completable text.
     */
    renderCompletion: (props: any) => React.ReactElement<any, any>;
    /**
     * Renders the placeholder text when the input value is empty.
     * If not provided, the placeholder will be rendered with a regular, low opacity text node.
     */
    renderPlaceholder?: (props: any) => React.ReactElement<any, any>;
    /**
     * External hook to
     */
    requestCompletion?: () => void;
    /**
     * A ref to the underlying contenteditable wrapper element. This can be used to focus the editor, select text, etc.
     */
    editorRef?: MutableRefObject<any>;
    /**
     * If true, the input value may contain newline characters. If false, the input value will be foreced to stay a single line.
     */
    multiline?: boolean;
    /**
     * Called when a non-empty completion is shown to the user.
     */
    onShowCompletion?: (completion: string) => void;
    /**
     * Called when a non-empty completion is accepted by the user.
     */
    onAcceptCompletion?: (completion: string) => void;
    /**
     * Called when the input loses focus.
     */
    onBlur?: React.FocusEventHandler<HTMLElement>;
    /**
     * Called when the input gains focus.
     */
    onFocus?: React.FocusEventHandler<HTMLElement>;
    /**
     * Called when a key is pressed while the input is focused.
     */
    onKeyDown?: React.KeyboardEventHandler<HTMLElement>;
    /**
     * The value to display when the input is empty. If not provided, no placeholder is displayed.
     */
    placeholder?: string;
    /**
     * Whether to allow the user to edit the input. If false, the input will be read-only.
     */
    disabled?: boolean;
    /**
     * Whether or not to request completions. If true, the input will not request completions and will behave like a normal input.
     * It will still collect data and learn to generate completions.
     */
    safeMode?: boolean;
    /**
     * The CSS styles to apply to the contenteditable div container.
     */
    style?: React.CSSProperties;
    /**
     * The CSS class name to apply to the contenteditable div container.
     */
    className?: string;
    /**
     * If provided, this function will be called to generate a completion instead of fetching one from the model.
     */
    getCompletion?: (text: string) => string;
    /**
     * If trye, the component will make requests to localhost instead of default API URL.
     */
    dev?: boolean;
    /**
     * The provided, the element to use for the container. Defaults to a `<div>`.
     */
    container?: React.ElementType<P>;
    /**
     * Props to pass to the container element, when one is provided.
     */
    containerProps?: P;
}
/**
 * A text input that provides intelligent, tab-completable completions based on the text that has already been entered.
 * Due to the limitations of `<input>` and `<textarea>` elements, this component renders a `<div contenteditable="true">` instead.
 */
declare function SmartInput<P>(p: SmartInputProps<P>): JSX.Element;

export { SmartInput, useSmartInput };
